; ********************************************************
; * Routinen zur Zahlumwandlung, Version 0.1 Januar 2002 *
; * (C)2002 by info@avr-asm-tutorial.net                 *
; ********************************************************
;
; Die folgenden Regeln gelten fr alle Routinen zur Zahl-
; umwandlung:
; - Fehler whrend der Umwandlung werden durch ein gesetz-
;   tes T-Bit im Status-Register signalisiert.
; - Der Z Zeiger zeigt entweder in das SRAM (Adresse >=
;   $0060) oder auf einen Registerbereich (Adressen $0000
;   bis $001D), die Register R0, R16 und R30/31 drfen
;   nicht in dem benutzten Bereich liegen!
; - ASCII- und BCD-kodierte mehrstellige Zahlen sind ab-
;   steigend geordnet, d.h. die hherwertigen Ziffern ha-
;   ben die niedrigerwertigen Adressen.
; - 16-bit-Binrzahlen sind generell in den Registern
;   rBin1H:rBin1L lokalisiert, bei einigen Routinen wird
;   zustzlich rBin2H:rBin2L verwendet. Diese mssen im
;   Hauptprogramm definiert werden.
; - Bei Binrzahlen ist die Lage im Registerbereich nicht
;   magebend, sie knnen auf- oder absteigend geordnet
;   sein oder getrennt im Registerraum liegen. Zu vernei-
;   den ist eine Zuordnung zu R0, rmp, ZH oder ZL.
; - Register rmp (Bereich: R16..R29) wird innerhalb der
;   Rechenroutinen benutzt, sein Inhalt ist nach Rckkehr
;   nicht definiert.
; - Das Registerpaar Z wird innerhalb von Routinen verwen-
;   det. Bei der Rckkehr ist sein Inhalt abhngig vom
;   Fehlerstatus definiert.
; - Einige Routinen verwenden zeitweise Register R0. Sein
;   Inhalt wird vor der Rckkehr wieder hergestellt.
; - Wegen der Verwendung des Z-Registers ist in jedem Fall
;   die Headerdatei des Prozessors einzubinden oder ZL
;   (R30) und ZH (R31) sind manuell zu definieren. Wird
;   die Headerdatei oder die manuelle Definition nicht
;   vorgenommen, gibt es beim Assemblieren eine Fehlermel-
;   dung oder es geschehen rtselhafte Dinge.
; - Wegen der Verwendung von Unterroutinen muss der Sta-
;   pelzeiger (SPH:SPL bzw. SPL bei <256 Byte SRAM)
;   initiiert sein.
;
; ************* berblick ber die Routinen **************
; Routine    Aufruf       Bedingungen          Rckkehr,
;                                              Fehler
; --------------------------------------------------------
; AscToBin2  Z zeigt auf  Beendet beim ersten  16-bit-Bin
;            erstes       Zeichen, das nicht   rBin1H:L,
;            ASCII-       einer Dezimalziffer  berlauf-
;            Zeichen      entspricht, ber-    fehler
;                         liest Leerzeichen
;                         und fhrende Nullen
; Asc5ToBin2 Z zeigt auf  Bentigt exakt 5     16-bit-Bin
;            erstes       gltige Ziffern,     rBin1H:L,
;            ASCII-       berliest Leerzei-   berlauf
;            Zeichen      chen und Nullen      oder ungl-
;                                              tige Ziffern
; Bcd5ToBin2 Z zeigt auf  Bentigt exakt 5     16-bit-Bin
;            5-stellige   gltige Ziffern      rBin1H:L
;            BCD-Zahl                          berlauf
;                                              oder ungl-
;                                              tige Ziffern 
; Bin2ToBcd5 16-bit-Bin   Z zeigt auf erste    5-digit-BCD
;            in rBin1H:L   BCD-Ziffer (auch     ab Z, keine
;                         nach Rckkehr)       Fehler   
; Bin2ToHex4 16-bit-Bin   Z zeigt auf erste    4-stellige 
;            in rBin1H:L  Hex-ASCII-Stelle,    Hex-Zahl ab
;                         Ausgabe A...F        Z, keine
;                                              Fehler
; Hex4ToBin2 4-digit-Hex  Bentigt exakt vier  16-bit-Bin
;            Z zeigt auf  Stellen Hex-ASCII,   rBin1H:L,
;            erste Stelle akzeptiert A...F und ungltige
;                         a...f                Hex-Ziffer
;
; ******************* Umwandlungscode ********************
;
; Paket I: Von ASCII bzw. BCD nach Binr
;
; AscToBin2
; =========
; wandelt eine ASCII-kodierte Zahl in eine 2-Byte-/16-Bit-
;   Binrzahl um.
; Aufruf: Z zeigt auf erste Stelle der umzuwandelnden
;   Zahl, die Umwandlung wird bei der ersten nicht dezima-
;   len Ziffer beendet.
; Stellen: Zulssig sind alle Zahlen, die innerhalb des
;   Wertebereiches von 16 Bit binr liegen (0..65535).
; Rckkehr: Gesetztes T-Flag im Statusregister zeigt Feh-
;   ler an.
;   T=0: Zahl in rBin1H:L ist gltig, Z zeigt auf erstes
;     Zeichen, das keiner Dezimalziffer entsprach
;   T=1: berlauffehler (Zahl zu gro), rBin1H:L undefi-
;     niert, Z zeigt auf Zeichen, bei dessen Verarbeitung
;     der Fehler auftrat.
; Bentigte Register: rBin1H:L (Ergebnis), rBin2H:L (wieder
;   hergestellt), rmp
; Bentigte Unterroutinen: Bin1Mul10
;
AscToBin2:
	clr rBin1H ; Ergebnis auf Null setzen
	clr rBin1L
	clt ; Fehlerflagge zurcksetzen
AscToBin2a:
	ld rmp,Z+ ; lese Zeichen
	cpi rmp,' ' ; ignoriere fhrende Leerzeichen ...
	breq AscToBin2a
	cpi rmp,'0' ; ... und Nullen
	breq AscToBin2a
AscToBin2b:
	subi rmp,'0' ; Subtrahiere ASCII-Null
	brcs AscToBin2d ; Ende der Zahl erkannt
	cpi rmp,10 ; prfe Ziffer
	brcc AscToBin2d ; Ende der Umwandlung
	rcall Bin1Mul10 ; Binrzahl mit 10 malnehmen
	brts AscToBin2c ; berlauf, gesetztes T-Flag
	add rBin1L,rmp ; Addiere Ziffer zur Binrzahl
	ld rmp,Z+ ; Lese schon mal nchstes Zeichen
	brcc AscToBin2b ; Kein berlauf ins MSB
	inc rBin1H ; berlauf ins nchste Byte
	brne AscToBin2b ; Kein berlauf ins nchste Byte
	set ; Setze berlauffehler-Flagge
AscToBin2c:
	sbiw ZL,1 ; berlauf trat bei letztem Zeichen auf
AscToBin2d:
	ret ; fertig, Rckkehr
;
; Asc5ToBin2
; ==========
; wandelt eine fnfstellige ASCII-kodierte Zahl in 2-Byte-
; Binrzahl um.
; Aufruf: Z zeigt auf erste Stelle der ASCII-kodierten
;   Zahl, fhrende Leerzeichen und Nullen sind erlaubt.
; Stellen: Die Zahl muss exakt 5 gltige Stellen haben.
; Rckkehr: T-Flag zeigt Fehlerbedingung an:
;   T=0: Binrzahl in rBin1H:L ist gltig, Z zeigt auf
;     erste Stelle der ASCII-kodierten Zahl.
;   T=1: Fehler bei der Umwandlung. Entweder war die Zahl
;     zu gro (0..65535, Z zeigt auf die Ziffer, bei der
;     der berlauf auftrat) oder sie enthilt ein unglti-
;     ges Zeichen (Z zeigt auf das ungltige Zeichen).
; Bentigte Register: rBin1H:L (Ergebnis), R0 (wiederher-
;   gestellt), rBin2H:L (wieder hergestellt), rmp
; Aufgerufene Unterroutinen: Bin1Mul10
;
Asc5ToBin2:
	push R0 ; R0 wird als Zhler verwendet, retten
	ldi rmp,6 ; Fnf Ziffern, einer zu viel
	mov R0,rmp ; in Zhlerregister R0
	clr rBin1H ; Ergebnis auf Null setzen
	clr rBin1L
	clt ; Fehlerflagge T-Bit zurcksetzen
Asc5ToBin2a:
	dec R0 ; Alle Zeichen leer oder Null?
	breq Asc5ToBin2d ; Ja, beenden
	ld rmp,Z+ ; Lese nchstes Zeichen
	cpi rmp,' ' ; berlese Leerzeichen
	breq Asc5ToBin2a ; geh zum nchsten Zeichen
	cpi rmp,'0' ; berlese fhrende Nullen
	breq Asc5ToBin2a ; geh zum nchsten Zeichen
Asc5ToBin2b:
	subi rmp,'0' ; Behandle Ziffer, ziehe ASCII-0 ab
	brcs Asc5ToBin2e ; Ziffer ist ungltig, raus
	cpi rmp,10 ; Ziffer grer als neun?
	brcc Asc5ToBin2e ; Ziffer ist ungltig, raus
	rcall Bin1Mul10 ; Multipliziere Binrzahl mit 10
	brts Asc5ToBin2e ; berlauf, raus
	add rBin1L,rmp ; addiere die Ziffer zur Binrzahl
	ld rmp,z+ ; lese schon mal das nchste Zeichen
	brcc Asc5ToBin2c ; Kein berlauf in das nchste Byte
	inc rBin1H ; berlauf in das nchste Byte
	breq Asc5ToBin2e ; berlauf auch ins bernchste Byte
Asc5ToBin2c:
	dec R0 ; Verringere Zhler fr Anzahl Zeichen
	brne Asc5ToBin2b ; Wandle weitere Zeichen um
Asc5ToBin2d: ; Ende der ASCII-kodierten Zahl erreicht
	sbiw ZL,5 ; Stelle die Startposition in Z wieder her
	pop R0 ; Stelle Register R0 wieder her
	ret ; Kehre zurck
Asc5ToBin2e: ; Letztes Zeichen war ungltig
	sbiw ZL,1 ; Zeige mit Z auf ungltiges Zeichen
	pop R0 ; Stelle Register R0 wieder her
	set ; Setze T-Flag fr Fehler
	ret ; und kejre zurck
;
; Bcd5ToBin2
; ==========
; wandelt eine 5-bit-BCD-Zahl in eine 16-Bit-Binrzahl um
; Aufruf: Z zeigt auf erste Stelle der BCD-kodierten
;   Zahl
; Stellen: Die Zahl muss exakt 5 gltige Stellen haben.
; Rckkehr: T-Flag zeigt Fehlerbedingung an:
;   T=0: Binrzahl in rBin1H:L ist gltig, Z zeigt auf
;     erste Stelle der BCD-kodierten Zahl.
;   T=1: Fehler bei der Umwandlung. Entweder war die Zahl
;     zu gro (0..65535, Z zeigt auf die Ziffer, bei der
;     der berlauf auftrat) oder sie enthielt ein unglti-
;     ges Zeichen (Z zeigt auf das ungltige Zeichen).
; Bentigte Register: rBin1H:L (Ergebnis), R0 (wiederher-
;   gestellt), rBin2H:L (wieder hergestellt), rmp
; Aufgerufene Unterroutinen: Bin1Mul10
;
Bcd5ToBin2:
	push R0 ; Rette Register R0
	clr rBin1H ; Setze Ergebnis Null
	clr rBin1L
	ldi rmp,5 ; Setze Zhler auf 5 Ziffern
	mov R0,rmp ; R0 ist Zhler
	clt ; Setze Fehlerflagge zurck
Bcd5ToBin2a:
	ld rmp,Z+ ; Lese BCD-Ziffer
	cpi rmp,10 ; prfe ob Ziffer korrekt
	brcc Bcd5ToBin2c ; ungltige BCD-Ziffer
	rcall Bin1Mul10 ; Multipliziere Ergebnis mit 10
	brts Bcd5ToBin2c ; berlauf aufgetreten
	add rBin1L,rmp ; Addiere Ziffer
	brcc Bcd5ToBin2b ; Kein berlauf ins nchste Byte
	inc rBin1H ; berlauf ins nchste Byte
	breq Bcd5ToBin2c ; berlauf ins bernchste Byte
Bcd5ToBin2b:
	dec R0 ; weitere Ziffer?
	brne Bcd5ToBin2a ; Ja
	pop R0 ; Stelle Register wieder her
	sbiw ZL,5 ; Setze Zeiger auf erste Stelle der BCD-Zahl
	ret ; Kehre zurck
Bcd5ToBin2c:
	sbiw ZL,1 ; Eine Ziffer zurck
	pop R0 ; Stelle Register wieder her
	set ; Setze T-flag, Fehler
	ret ; Kehre zurck
;
; Bin1Mul10
; =========
; Multipliziert die 16-Bit-Binrzahl mit 10
; Unterroutine benutzt von AscToBin2, Asc5ToBin2, Bcd5ToBin2
; Aufruf: 16-Bit-Binrzahl in rBin1H:L
; Rckkehr: T-Flag zeigt gltiges Ergebnis an.
;   T=0: rBin1H:L enthlt gltiges Ergebnis.
;   T=1: berlauf bei der Multiplikation, rBin1H:L undefiniert
; Benutzte Register: rBin1H:L (Ergebnis), rBin2H:L (wird wieder
;   hergestellt)
;
Bin1Mul10:
	push rBin2H ; Rette die Register rBin2H:L
	push rBin2L
	mov rBin2H,rBin1H ; Kopiere die Zahl dort hin
	mov rBin2L,rBin1L
	add rBin1L,rBin1L ; Multipliziere Zahl mit 2
	adc rBin1H,rBin1H
	brcs Bin1Mul10b ; berlauf, raus hier!
Bin1Mul10a:
	add rBin1L,rbin1L ; Noch mal mit 2 malnehmen (=4*Zahl)
	adc rBin1H,rBin1H
	brcs Bin1Mul10b ; berlauf, raus hier!
	add rBin1L,rBin2L ; Addiere die Kopie (=5*Zahl)
	adc rBin1H,rBin2H
	brcs Bin1Mul10b ;berlauf, raus hier!
	add rBin1L,rBin1L ; Noch mal mit 2 malnehmen (=10*Zahl)
	adc rBin1H,rBin1H
	brcc Bin1Mul10c ; Kein berlauf, berspringe
Bin1Mul10b:
	set ; berlauf, setze T-Flag
Bin1Mul10c:
	pop rBin2L ; Stelle die geretteten Register wieder her
	pop rBin2H
	ret ; Kehre zurck
;
; ************************************************
;
; Paket II: Von Binr nach ASCII bzw. BCD
;
; Bin2ToAsc5
; ==========
; wandelt eine 16-Bit-Binrzahl in eine fnfstellige ASCII-
;   kodierte Dezimalzahl um
; Aufruf: 16-Bit-Binrzahl in rBin1H:L, Z zeigt auf Anfang
;   der Zahl
; Rckkehr: Z zeigt auf Anfang der Zahl, fhrende Nullen sind
;   mit Leerzeichen berschrieben
; Benutzte Register: rBin1H:L (bleibt erhalten), rBin2H:L
;   (wird berschrieben), rmp
; Aufgerufene Unterroutinen: Bin2ToBcd5
;
Bin2ToAsc5:
	rcall Bin2ToBcd5 ; wandle Binrzahl in BCD um
	ldi rmp,4 ; Zhler auf 4
	mov rBin2L,rmp
Bin2ToAsc5a:
	ld rmp,z ; Lese eine BCD-Ziffer
	tst rmp ; prfe ob Null
	brne Bin2ToAsc5b ; Nein, erste Ziffer <> 0 gefunden
	ldi rmp,' ' ; mit Leerzeichen berschreiben
	st z+,rmp ; und ablegen
	dec rBin2L ; Zhler um eins senken
	brne Bin2ToAsc5a ; weitere fhrende Leerzeichen
	ld rmp,z ; Lese das letzte Zeichen
Bin2ToAsc5b:
	inc rBin2L ; Ein Zeichen mehr
Bin2ToAsc5c:
	subi rmp,-'0' ; Addiere ASCII-0
	st z+,rmp ; und speichere ab, erhhe Zeiger
	ld rmp,z ; nchstes Zeichen lesen
	dec rBin2L ; noch Zeichen behandeln?
	brne Bin2ToAsc5c ; ja, weitermachen
	sbiw ZL,5 ; Zeiger an Anfang
	ret ; fertig
;
; Bin2ToAsc
; =========
; wandelt eine 16-Bit-Binrzahl in eine fnfstellige ASCII-
;   kodierte Dezimalzahl um, Zeiger zeigt auf die erste signi-
;   fikante Ziffer der Zahl, und gibt Anzahl der Ziffern zu-
;   rck
; Aufruf: 16-Bit-Binrzahl in rBin1H:L, Z zeigt auf Anfang
;   der Zahl (5 Stellen erforderlich, auch bei kleineren Zah-
;   len!)
; Rckkehr: Z zeigt auf erste signifikante Ziffer der ASCII-
;   kodierten Zahl, rBin2L enthlt Lnge der Zahl (1..5)
; Benutzte Register: rBin1H:L (bleibt erhalten), rBin2H:L
;   (wird berschrieben), rmp
; Aufgerufene Unterroutinen: Bin2ToBcd5, Bin2ToAsc5
;
Bin2ToAsc:
	rcall Bin2ToAsc5 ; Wandle Binrzahl in ASCII
	ldi rmp,6 ; Zhler auf 6
	mov rBin2L,rmp
Bin2ToAsca:
	dec rBin2L ; verringere Zhler
	ld rmp,z+ ; Lese Zeichen und erhhe Zeiger
	cpi rmp,' ' ; war Leerzeichen?
	breq Bin2ToAsca ; Nein, war nicht
	sbiw ZL,1 ; ein Zeichen rckwrts
	ret ; fertig
;
; Bin2ToBcd5
; ==========
; wandelt 16-Bit-Binrzahl in 5-stellige BCD-Zahl um
; Aufruf: 16-Bit-Binrzahl in rBin1H:L, Z zeigt auf die
;   erste Stelle der BCD-kodierten Resultats
; Stellen: Die BCD-Zahl hat exakt 5 gltige Stellen.
; Rckkehr: Z zeigt auf die hchste BCD-Stelle
; Bentigte Register: rBin1H:L (wird erhalten), rBin2H:L
;   (wird nicht wieder hergestellt), rmp
; Aufgerufene Unterroutinen: Bin2ToDigit
;
Bin2ToBcd5:
	push rBin1H ; Rette Inhalt der Register rBin1H:L
	push rBin1L
	ldi rmp,HIGH(10000) ; Lade 10.000 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(10000)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 5.Stelle durch Abziehen
	ldi rmp,HIGH(1000) ; Lade 1.000 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(1000)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 4.Stelle durch Abziehen
	ldi rmp,HIGH(100) ; Lade 100 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(100)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 3.Stelle durch Abziehen
	ldi rmp,HIGH(10) ; Lade 10 in rBin2H:L
	mov rBin2H,rmp
	ldi rmp,LOW(10)
	mov rBin2L,rmp
	rcall Bin2ToDigit ; Ermittle 2.Stelle durch Abziehen
	st z,rBin1L ; Rest sind Einer
	sbiw ZL,4 ; Setze Zeiger Z auf 5.Stelle (erste Ziffer)
	pop rBin1L ; Stelle den Originalwert wieder her
	pop rBin1H
	ret ; und kehre zurck
;
; Bin2ToDigit
; ===========
; ermittelt eine dezimale Ziffer durch fortgesetztes Abziehen
; einer binr kodierten Dezimalstelle
; Unterroutine benutzt von: Bin2ToBcd5, Bin2ToAsc5, Bin2ToAsc
; Aufruf: Binrzahl in rBin1H:L, binr kodierte Dezimalzahl
;   in rBin2H:L, Z zeigt auf bearbeitete BCD-Ziffer
; Rckkehr: Ergebis in Z (bei Aufruf), Z um eine Stelle er-
;   hht, keine Fehlerbehandlung
; Benutzte Register: rBin1H:L (enthlt Rest der Binrzahl),
;   rBin2H (bleibt erhalten), rmp
; Aufgerufene Unterroutinen: -
;
Bin2ToDigit:
	clr rmp ; Zhler auf Null
Bin2ToDigita:
	cp rBin1H,rBin2H ; Vergleiche MSBs miteinander
	brcs Bin2ToDigitc ; MSB Binrzahl kleiner, fertig
	brne Bin2ToDigitb ; MSB Binrzahl grer, subtrahiere
	cp rBin1L,rBin2L ; MSB gleich, vergleiche LSBs
	brcs Bin2ToDigitc ; LSB Binrzahl kleiner, fertig
Bin2ToDigitb:
	sub rBin1L,rBin2L ; Subtrahiere LSB Dezimalzahl
	sbc rBin1H,rBin2H ; Subtrahiere Carry und MSB
	inc rmp ; Erhhe den Zhler
	rjmp Bin2ToDigita ; Weiter vergleichen/subtrahieren
Bin2ToDigitc:
	st z+,rmp ; Speichere das Ergebnis und erhhe Zeiger
	ret ; zurck
;
; ***********************************************
;
; Paket III: Von Binr nach Hex-ASCII
;
; Bin2ToHex4
; ==========
; wandelt eine 16-Bit-Binrzahl in Hex-ASCII
; Aufruf: Binrzahl in rBin1H:L, Z zeigt auf erste Position
;   des vierstelligen ASCII-Hex
; Rckkehr: Z zeigt auf erste Position des vierstelligen
;   ASCII-Hex, ASCII-Ziffern A..F in Grobuchstaben
; Benutzte Register: rBin1H:L (bleibt erhalten), rmp
; Aufgerufene Unterroutinen: Bin1ToHex2, Bin1ToHex1
;
Bin2ToHex4:
	mov rmp,rBin1H ; MSB in rmp kopieren
	rcall Bin1ToHex2 ; in Hex-ASCII umwandeln
	mov rmp,rBin1L ; LSB in rmp kopieren
	rcall Bin1ToHex2 ; in Hex-ASCII umwandeln
	sbiw ZL,4 ; Zeiger auf Anfang Hex-ASCII
	ret ; fertig
;
; Bin1ToHex2 wandelt die 8-Bit-Binrzahl in rmp in Hex-ASCII
; gehrt zu: Bin2ToHex4
;
Bin1ToHex2:
	push rmp ; Rette Byte auf dem Stapel
	swap rmp ; Vertausche die oberen und unteren 4 Bit
	rcall Bin1ToHex1 ; wandle untere 4 Bits in Hex-ASCII
	pop rmp ; Stelle das Byte wieder her
Bin1ToHex1:
	andi rmp,$0F ; Maskiere die oberen vier Bits
	subi rmp,-'0' ; Addiere ASCII-0
	cpi rmp,'9'+1 ; Ziffern A..F?
	brcs Bin1ToHex1a ; Nein
	subi rmp,-7 ; Addiere 7 fr A..F
Bin1ToHex1a:
	st z+,rmp ; abspeichern und Zeiger erhhen
	ret ; fertig
;
; ***********************************************
;
; Paket IV: Von Hex-ASCII nach Binr
;
; Hex4ToBin2
; ==========
; wandelt eine vierstellige Hex-ASCII-Zahl in eine 16-Bit-
;   Binrzahl um
; Aufruf: Z zeigt auf die erste Stelle der Hex-ASCII-Zahl
; Rckkehr: T-Flag zeigt Fehler an:
;   T=0: rBin1H:L enthlt die 16-Bit-Binrzahl, Z zeigt
;     auf die erste Hex-ASCII-Ziffer wie beim Aufruf
;   T=1: ungltige Hex-ASCII-Ziffer, Z zeigt auf ungltige
;     Ziffer
; Benutzte Register: rBin1H:L (enthlt Ergebnis), R0 (wie-
;   der hergestellt), rmp
; Aufgerufene Unterroutinen: Hex2ToBin1, Hex1ToBin1
;
Hex4ToBin2:
	clt ; Lsche Fehlerflag
	rcall Hex2ToBin1 ; Wandle zwei Hex-ASCII-Ziffern
	brts Hex4ToBin2a ; Fehler, beende hier
	mov rBin1H,rmp ; kopiere nach MSB Ergebnis
	rcall Hex2ToBin1 ; Wandle zwei Hex-ASCII-Ziffern
	brts Hex4ToBin2a ; Fehler, beende hier
	mov rBin1L,rmp ; kopiere nach LSB Ergebnis
	sbiw ZL,4 ; Ergebis ok, Zeiger auf Anfang
Hex4ToBin2a:
	ret ; zurck
;
; Hex2ToBin1 wandelt 2-stellig-Hex-ASCII nach 8-Bit-Binr
;
Hex2ToBin1:
	push R0 ; Rette Register R0
	rcall Hex1ToBin1 ; Wandle nchstes Zeichen in Byte
	brts Hex2ToBin1a ; Fehler, stop hier
	swap rmp; untere vier Bits in obere vier Bits
	mov R0,rmp ; zwischenspeichern
	rcall Hex1ToBin1 ; Nchstes Zeichen umwandeln
	brts Hex2ToBin1a ; Fehler, raus hier
	or rmp,R0 ; untere und obere vier Bits zusammen
Hex2ToBin1a:
	pop R0 ; Stelle R0 wieder her
	ret ; zurck
;
; Hex1ToBin1 liest ein Zeichen und wandelt es in Binr um
;
Hex1ToBin1:
	ld rmp,z+ ; Lese Zeichen
	subi rmp,'0' ; Ziehe ASCII-0 ab
	brcs Hex1ToBin1b ; Fehler, kleiner als 0
	cpi rmp,10 ; A..F ; Ziffer grer als 9?
	brcs Hex1ToBin1c ; nein, fertig
	cpi rmp,$30 ; Kleinbuchstaben?
	brcs Hex1ToBin1a ; Nein
	subi rmp,$20 ; Klein- in Grossbuchstaben
Hex1ToBin1a:
	subi rmp,7 ; Ziehe 7 ab, A..F ergibt $0A..$0F
	cpi rmp,10 ; Ziffer kleiner $0A?
	brcs Hex1ToBin1b ; Ja, Fehler
	cpi rmp,16 ; Ziffer grer als $0F
	brcs Hex1ToBin1c ; Nein, Ziffer in Ordnung
Hex1ToBin1b: ; Error
	sbiw ZL,1 ; Ein Zeichen zurck
	set ; Setze Fehlerflagge
Hex1ToBin1c:
	ret ; Zurck
